@@ -454,7 +454,7 @@ impl LintCacheData {
                 CacheMessage {
                     rule,
                     body: msg.body().to_string(),
-                    suggestion: msg.suggestion().map(ToString::to_string),
+                    suggestion: msg.first_help_text().map(ToString::to_string),
                     range: msg.expect_range(),
                     parent: msg.parent(),
                     fix: msg.fix().cloned(),
