@@ -162,3 +162,86 @@ except Exception:
         exception("An error occurred")
     else:
         exception("An error occurred")
+
+# Test tuple exceptions
+try:
+    pass
+except (Exception,):
+    pass
+
+try:
+    pass
+except (Exception, ValueError):
+    pass
+
+try:
+    pass
+except (ValueError, Exception):
+    pass
+
+try:
+    pass
+except (ValueError, Exception) as e:
+    print(e)
+
+try:
+    pass
+except (BaseException, TypeError):
+    pass
+
+try:
+    pass
+except (TypeError, BaseException):
+    pass
+
+try:
+    pass
+except (Exception, BaseException):
+    pass
+
+try:
+    pass
+except (BaseException, Exception):
+    pass
+
+# Test nested tuples
+try:
+    pass
+except ((Exception, ValueError), TypeError):
+    pass
+
+try:
+    pass
+except (ValueError, (BaseException, TypeError)):
+    pass
+
+# Test valid tuple exceptions (should not trigger)
+try:
+    pass
+except (ValueError, TypeError):
+    pass
+
+try:
+    pass
+except (OSError, FileNotFoundError):
+    pass
+
+try:
+    pass
+except (OSError, FileNotFoundError) as e:
+    print(e)
+
+try:
+    pass
+except (Exception, ValueError):
+    critical("...", exc_info=True)
+
+try:
+    pass
+except (Exception, ValueError):
+    raise
+
+try:
+    pass
+except (Exception, ValueError) as e:
+    raise e
