@@ -1039,15 +1039,10 @@ pub(crate) fn expression(expr: &Expr, checker: &Checker) {
                 flake8_simplify::rules::zip_dict_keys_and_values(checker, call);
             }
             if checker.any_rule_enabled(&[
-                Rule::OsChmod,
                 Rule::OsMkdir,
                 Rule::OsMakedirs,
-                Rule::OsRename,
-                Rule::OsReplace,
-                Rule::OsGetcwd,
                 Rule::OsStat,
                 Rule::OsPathJoin,
-                Rule::OsPathSamefile,
                 Rule::OsPathSplitext,
                 Rule::BuiltinOpen,
                 Rule::PyPath,
@@ -1110,6 +1105,21 @@ pub(crate) fn expression(expr: &Expr, checker: &Checker) {
                 if checker.is_rule_enabled(Rule::OsReadlink) {
                     flake8_use_pathlib::rules::os_readlink(checker, call, segments);
                 }
+                if checker.is_rule_enabled(Rule::OsGetcwd) {
+                    flake8_use_pathlib::rules::os_getcwd(checker, call, segments);
+                }
+                if checker.is_rule_enabled(Rule::OsChmod) {
+                    flake8_use_pathlib::rules::os_chmod(checker, call, segments);
+                }
+                if checker.is_rule_enabled(Rule::OsRename) {
+                    flake8_use_pathlib::rules::os_rename(checker, call, segments);
+                }
+                if checker.is_rule_enabled(Rule::OsReplace) {
+                    flake8_use_pathlib::rules::os_replace(checker, call, segments);
+                }
+                if checker.is_rule_enabled(Rule::OsPathSamefile) {
+                    flake8_use_pathlib::rules::os_path_samefile(checker, call, segments);
+                }
                 if checker.is_rule_enabled(Rule::PathConstructorCurrentDirectory) {
                     flake8_use_pathlib::rules::path_constructor_current_directory(
                         checker, call, segments,
