@@ -56,13 +56,19 @@ impl<'a> Insertion<'a> {
         stylist: &Stylist,
     ) -> Insertion<'static> {
         // Skip over any docstrings.
-        let mut location = if let Some(location) = match_docstring_end(body) {
+        let mut location = if let Some(mut location) = match_docstring_end(body) {
             // If the first token after the docstring is a semicolon, insert after the semicolon as
             // an inline statement.
             if let Some(offset) = match_semicolon(locator.after(location)) {
                 return Insertion::inline(" ", location.add(offset).add(TextSize::of(';')), ";");
             }
 
+            // If the first token after the docstring is a continuation character (i.e. "\"), advance
+            // an additional row to prevent inserting in the same logical line.
+            if match_continuation(locator.after(location)).is_some() {
+                location = locator.full_line_end(location);
+            }
+
             // Otherwise, advance to the next row.
             locator.full_line_end(location)
         } else {
@@ -323,7 +329,7 @@ mod tests {
 
     #[test]
     fn start_of_file() -> Result<()> {
-        fn insert(contents: &str) -> Result<Insertion> {
+        fn insert(contents: &str) -> Result<Insertion<'_>> {
             let parsed = parse_module(contents)?;
             let locator = Locator::new(contents);
             let stylist = Stylist::from_tokens(parsed.tokens(), locator.contents());
@@ -363,6 +369,16 @@ mod tests {
             Insertion::own_line("", TextSize::from(20), "\n")
         );
 
+        let contents = r#"
+"""Hello, world!"""\
+
+"#
+        .trim_start();
+        assert_eq!(
+            insert(contents)?,
+            Insertion::own_line("", TextSize::from(22), "\n")
+        );
+
         let contents = r"
 x = 1
 "
@@ -434,7 +450,7 @@ x = 1
 
     #[test]
     fn start_of_block() {
-        fn insert(contents: &str, offset: TextSize) -> Insertion {
+        fn insert(contents: &str, offset: TextSize) -> Insertion<'_> {
             let parsed = parse_module(contents).unwrap();
             let locator = Locator::new(contents);
             let stylist = Stylist::from_tokens(parsed.tokens(), locator.contents());
