@@ -1,5 +1,7 @@
 pub(crate) use glob_rule::*;
 pub(crate) use invalid_pathlib_with_suffix::*;
+pub(crate) use os_chmod::*;
+pub(crate) use os_getcwd::*;
 pub(crate) use os_path_abspath::*;
 pub(crate) use os_path_basename::*;
 pub(crate) use os_path_dirname::*;
@@ -13,8 +15,11 @@ pub(crate) use os_path_isabs::*;
 pub(crate) use os_path_isdir::*;
 pub(crate) use os_path_isfile::*;
 pub(crate) use os_path_islink::*;
+pub(crate) use os_path_samefile::*;
 pub(crate) use os_readlink::*;
 pub(crate) use os_remove::*;
+pub(crate) use os_rename::*;
+pub(crate) use os_replace::*;
 pub(crate) use os_rmdir::*;
 pub(crate) use os_sep_split::*;
 pub(crate) use os_unlink::*;
@@ -23,6 +28,8 @@ pub(crate) use replaceable_by_pathlib::*;
 
 mod glob_rule;
 mod invalid_pathlib_with_suffix;
+mod os_chmod;
+mod os_getcwd;
 mod os_path_abspath;
 mod os_path_basename;
 mod os_path_dirname;
@@ -36,8 +43,11 @@ mod os_path_isabs;
 mod os_path_isdir;
 mod os_path_isfile;
 mod os_path_islink;
+mod os_path_samefile;
 mod os_readlink;
 mod os_remove;
+mod os_rename;
+mod os_replace;
 mod os_rmdir;
 mod os_sep_split;
 mod os_unlink;
