@@ -1,29 +1,43 @@
 mod completion;
+mod doc_highlights;
 mod docstring;
+mod document_symbols;
 mod find_node;
 mod goto;
 mod goto_declaration;
 mod goto_definition;
+mod goto_references;
 mod goto_type_definition;
 mod hover;
 mod inlay_hints;
 mod markup;
 mod references;
+mod rename;
+mod selection_range;
 mod semantic_tokens;
 mod signature_help;
 mod stub_mapping;
+mod symbols;
+mod workspace_symbols;
 
 pub use completion::completion;
+pub use doc_highlights::document_highlights;
 pub use docstring::get_parameter_documentation;
+pub use document_symbols::{document_symbols, document_symbols_with_options};
 pub use goto::{goto_declaration, goto_definition, goto_type_definition};
+pub use goto_references::goto_references;
 pub use hover::hover;
-pub use inlay_hints::inlay_hints;
+pub use inlay_hints::{InlayHintSettings, inlay_hints};
 pub use markup::MarkupKind;
-pub use references::references;
+pub use references::ReferencesMode;
+pub use rename::{can_rename, rename};
+pub use selection_range::selection_range;
 pub use semantic_tokens::{
     SemanticToken, SemanticTokenModifier, SemanticTokenType, SemanticTokens, semantic_tokens,
 };
 pub use signature_help::{ParameterDetails, SignatureDetails, SignatureHelpInfo, signature_help};
+pub use symbols::{SymbolInfo, SymbolKind, SymbolsOptions};
+pub use workspace_symbols::{WorkspaceSymbolInfo, workspace_symbols};
 
 use ruff_db::files::{File, FileRange};
 use ruff_text_size::{Ranged, TextRange};
@@ -110,6 +124,53 @@ impl NavigationTarget {
     }
 }
 
+/// Specifies the kind of reference operation.
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+pub enum ReferenceKind {
+    /// A read reference to a symbol (e.g., using a variable's value)
+    Read,
+    /// A write reference to a symbol (e.g., assigning to a variable)
+    Write,
+    /// Neither a read or a write (e.g., a function or class declaration)
+    Other,
+}
+
+/// Target of a reference with information about the kind of operation.
+/// Unlike `NavigationTarget`, this type is specifically designed for references
+/// and contains only a single range (not separate focus/full ranges) and
+/// includes information about whether the reference is a read or write operation.
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+pub struct ReferenceTarget {
+    file_range: FileRange,
+    kind: ReferenceKind,
+}
+
+impl ReferenceTarget {
+    /// Creates a new `ReferenceTarget`.
+    pub fn new(file: File, range: TextRange, kind: ReferenceKind) -> Self {
+        Self {
+            file_range: FileRange::new(file, range),
+            kind,
+        }
+    }
+
+    pub fn file(&self) -> File {
+        self.file_range.file()
+    }
+
+    pub fn range(&self) -> TextRange {
+        self.file_range.range()
+    }
+
+    pub fn file_range(&self) -> FileRange {
+        self.file_range
+    }
+
+    pub fn kind(&self) -> ReferenceKind {
+        self.kind
+    }
+}
+
 #[derive(Debug, Clone)]
 pub struct NavigationTargets(smallvec::SmallVec<[NavigationTarget; 1]>);
 
