@@ -173,21 +173,30 @@ impl<'a> ProjectFilesWalker<'a> {
                     Ok(entry) => {
                         // Skip excluded directories unless they were explicitly passed to the walker
                         // (which is the case passed to `ty check <paths>`).
-                        if entry.file_type().is_directory() && entry.depth() > 0 {
-                            return match self.filter.is_directory_included(entry.path(), GlobFilterCheckMode::TopDown) {
-                                IncludeResult::Included => WalkState::Continue,
-                                IncludeResult::Excluded => {
-                                    tracing::debug!("Skipping directory '{path}' because it is excluded by a default or `src.exclude` pattern", path=entry.path());
-                                   WalkState::Skip
-                                },
-                                IncludeResult::NotIncluded => {
-                                    tracing::debug!("Skipping directory `{path}` because it doesn't match any `src.include` pattern or path specified on the CLI", path=entry.path());
-                                    WalkState::Skip
-                                },
-                            };
-                        }
-
-                        if entry.file_type().is_file() {
+                        if entry.file_type().is_directory() {
+                            if entry.depth() > 0 {
+                                let directory_included = self
+                                    .filter
+                                    .is_directory_included(entry.path(), GlobFilterCheckMode::TopDown);
+                                return match directory_included {
+                                    IncludeResult::Included => WalkState::Continue,
+                                    IncludeResult::Excluded => {
+                                        tracing::debug!(
+                                            "Skipping directory '{path}' because it is excluded by a default or `src.exclude` pattern",
+                                            path=entry.path()
+                                        );
+                                        WalkState::Skip
+                                    },
+                                    IncludeResult::NotIncluded => {
+                                        tracing::debug!(
+                                            "Skipping directory `{path}` because it doesn't match any `src.include` pattern or path specified on the CLI",
+                                            path=entry.path()
+                                        );
+                                        WalkState::Skip
+                                    },
+                                };
+                            }
+                        } else {
                             // Ignore any non python files to avoid creating too many entries in `Files`.
                             if entry
                                 .path()
@@ -201,14 +210,23 @@ impl<'a> ProjectFilesWalker<'a> {
                             // For all files, except the ones that were explicitly passed to the walker (CLI),
                             // check if they're included in the project.
                             if entry.depth() > 0 {
-                                match self.filter.is_file_included(entry.path(), GlobFilterCheckMode::TopDown) {
+                                match self
+                                    .filter
+                                    .is_file_included(entry.path(), GlobFilterCheckMode::TopDown)
+                                {
                                     IncludeResult::Included => {},
                                     IncludeResult::Excluded => {
-                                        tracing::debug!("Ignoring file `{path}` because it is excluded by a default or `src.exclude` pattern.", path=entry.path());
+                                        tracing::debug!(
+                                            "Ignoring file `{path}` because it is excluded by a default or `src.exclude` pattern.",
+                                            path=entry.path()
+                                        );
                                         return WalkState::Continue;
                                     },
                                     IncludeResult::NotIncluded => {
-                                        tracing::debug!("Ignoring file `{path}` because it doesn't match any `src.include` pattern or path specified on the CLI.", path=entry.path());
+                                        tracing::debug!(
+                                            "Ignoring file `{path}` because it doesn't match any `src.include` pattern or path specified on the CLI.",
+                                            path=entry.path()
+                                        );
                                         return WalkState::Continue;
                                     },
                                 }
