@@ -20,23 +20,27 @@ use crate::semantic_index::builder::SemanticIndexBuilder;
 use crate::semantic_index::definition::{Definition, DefinitionNodeKey, Definitions};
 use crate::semantic_index::expression::Expression;
 use crate::semantic_index::narrowing_constraints::ScopedNarrowingConstraint;
-use crate::semantic_index::place::{
-    FileScopeId, NodeWithScopeKey, NodeWithScopeRef, PlaceExpr, PlaceTable, Scope, ScopeId,
-    ScopeKind, ScopedPlaceId,
+use crate::semantic_index::place::{PlaceExprRef, PlaceTable};
+pub use crate::semantic_index::scope::FileScopeId;
+use crate::semantic_index::scope::{
+    NodeWithScopeKey, NodeWithScopeRef, Scope, ScopeId, ScopeKind, ScopeLaziness,
 };
-use crate::semantic_index::use_def::{EagerSnapshotKey, ScopedEagerSnapshotId, UseDefMap};
+use crate::semantic_index::symbol::ScopedSymbolId;
+use crate::semantic_index::use_def::{EnclosingSnapshotKey, ScopedEnclosingSnapshotId, UseDefMap};
 use crate::semantic_model::HasTrackedScope;
-use crate::util::get_size::untracked_arc_size;
 
 pub mod ast_ids;
 mod builder;
 pub mod definition;
 pub mod expression;
+pub(crate) mod member;
 pub(crate) mod narrowing_constraints;
 pub mod place;
 pub(crate) mod predicate;
 mod re_exports;
 mod reachability_constraints;
+pub(crate) mod scope;
+pub(crate) mod symbol;
 mod use_def;
 
 pub(crate) use self::use_def::{
@@ -44,12 +48,10 @@ pub(crate) use self::use_def::{
     DeclarationWithConstraint, DeclarationsIterator,
 };
 
-type PlaceSet = hashbrown::HashTable<ScopedPlaceId>;
-
 /// Returns the semantic index for `file`.
 ///
 /// Prefer using [`symbol_table`] when working with symbols from a single scope.
-#[salsa::tracked(returns(ref), no_eq, heap_size=get_size2::GetSize::get_heap_size)]
+#[salsa::tracked(returns(ref), no_eq, heap_size=ruff_memory_usage::heap_size)]
 pub(crate) fn semantic_index(db: &dyn Db, file: File) -> SemanticIndex<'_> {
     let _span = tracing::trace_span!("semantic_index", ?file).entered();
 
@@ -63,7 +65,7 @@ pub(crate) fn semantic_index(db: &dyn Db, file: File) -> SemanticIndex<'_> {
 /// Using [`place_table`] over [`semantic_index`] has the advantage that
 /// Salsa can avoid invalidating dependent queries if this scope's place table
 /// is unchanged.
-#[salsa::tracked(returns(deref), heap_size=get_size2::GetSize::get_heap_size)]
+#[salsa::tracked(returns(deref), heap_size=ruff_memory_usage::heap_size)]
 pub(crate) fn place_table<'db>(db: &'db dyn Db, scope: ScopeId<'db>) -> Arc<PlaceTable> {
     let file = scope.file(db);
     let _span = tracing::trace_span!("place_table", scope=?scope.as_id(), ?file).entered();
@@ -83,7 +85,7 @@ pub(crate) fn place_table<'db>(db: &'db dyn Db, scope: ScopeId<'db>) -> Arc<Plac
 ///
 ///   - We cannot resolve relative imports (which aren't allowed in `import` statements) without
 ///     knowing the name of the current module, and whether it's a package.
-#[salsa::tracked(returns(deref), heap_size=get_size2::GetSize::get_heap_size)]
+#[salsa::tracked(returns(deref), heap_size=ruff_memory_usage::heap_size)]
 pub(crate) fn imported_modules<'db>(db: &'db dyn Db, file: File) -> Arc<FxHashSet<ModuleName>> {
     semantic_index(db, file).imported_modules.clone()
 }
@@ -93,8 +95,8 @@ pub(crate) fn imported_modules<'db>(db: &'db dyn Db, file: File) -> Arc<FxHashSe
 /// Using [`use_def_map`] over [`semantic_index`] has the advantage that
 /// Salsa can avoid invalidating dependent queries if this scope's use-def map
 /// is unchanged.
-#[salsa::tracked(returns(deref), heap_size=get_size2::GetSize::get_heap_size)]
-pub(crate) fn use_def_map<'db>(db: &'db dyn Db, scope: ScopeId<'db>) -> ArcUseDefMap<'db> {
+#[salsa::tracked(returns(deref), heap_size=ruff_memory_usage::heap_size)]
+pub(crate) fn use_def_map<'db>(db: &'db dyn Db, scope: ScopeId<'db>) -> Arc<UseDefMap<'db>> {
     let file = scope.file(db);
     let _span = tracing::trace_span!("use_def_map", scope=?scope.as_id(), ?file).entered();
     let index = semantic_index(db, file);
@@ -117,10 +119,10 @@ pub(crate) fn attribute_assignments<'db, 's>(
 
     attribute_scopes(db, class_body_scope).filter_map(|function_scope_id| {
         let place_table = index.place_table(function_scope_id);
-        let place = place_table.place_id_by_instance_attribute_name(name)?;
+        let member = place_table.member_id_by_instance_attribute_name(name)?;
         let use_def = &index.use_def_maps[function_scope_id];
         Some((
-            use_def.inner.all_reachable_bindings(place),
+            use_def.all_reachable_member_bindings(member),
             function_scope_id,
         ))
     })
@@ -141,10 +143,10 @@ pub(crate) fn attribute_declarations<'db, 's>(
 
     attribute_scopes(db, class_body_scope).filter_map(|function_scope_id| {
         let place_table = index.place_table(function_scope_id);
-        let place = place_table.place_id_by_instance_attribute_name(name)?;
+        let member = place_table.member_id_by_instance_attribute_name(name)?;
         let use_def = &index.use_def_maps[function_scope_id];
         Some((
-            use_def.inner.all_reachable_declarations(place),
+            use_def.all_reachable_member_declarations(member),
             function_scope_id,
         ))
     })
@@ -181,14 +183,14 @@ pub(crate) fn attribute_scopes<'db, 's>(
 }
 
 /// Returns the module global scope of `file`.
-#[salsa::tracked(heap_size=get_size2::GetSize::get_heap_size)]
+#[salsa::tracked(heap_size=ruff_memory_usage::heap_size)]
 pub(crate) fn global_scope(db: &dyn Db, file: File) -> ScopeId<'_> {
     let _span = tracing::trace_span!("global_scope", ?file).entered();
 
     FileScopeId::global().to_scope_id(db, file)
 }
 
-pub(crate) enum EagerSnapshotResult<'map, 'db> {
+pub(crate) enum EnclosingSnapshotResult<'map, 'db> {
     FoundConstraint(ScopedNarrowingConstraint),
     FoundBindings(BindingWithConstraintsIterator<'map, 'db>),
     NotFound,
@@ -220,7 +222,7 @@ pub(crate) struct SemanticIndex<'db> {
     scope_ids_by_scope: IndexVec<FileScopeId, ScopeId<'db>>,
 
     /// Use-def map for each scope in this file.
-    use_def_maps: IndexVec<FileScopeId, ArcUseDefMap<'db>>,
+    use_def_maps: IndexVec<FileScopeId, Arc<UseDefMap<'db>>>,
 
     /// Lookup table to map between node ids and ast nodes.
     ///
@@ -234,8 +236,8 @@ pub(crate) struct SemanticIndex<'db> {
     /// Flags about the global scope (code usage impacting inference)
     has_future_annotations: bool,
 
-    /// Map of all of the eager snapshots that appear in this file.
-    eager_snapshots: FxHashMap<EagerSnapshotKey, ScopedEagerSnapshotId>,
+    /// Map of all of the enclosing snapshots that appear in this file.
+    enclosing_snapshots: FxHashMap<EnclosingSnapshotKey, ScopedEnclosingSnapshotId>,
 
     /// List of all semantic syntax errors in this file.
     semantic_syntax_errors: Vec<SemanticSyntaxError>,
@@ -259,7 +261,7 @@ impl<'db> SemanticIndex<'db> {
     /// Use the Salsa cached [`use_def_map()`] query if you only need the
     /// use-def map for a single scope.
     #[track_caller]
-    pub(super) fn use_def_map(&self, scope_id: FileScopeId) -> ArcUseDefMap<'_> {
+    pub(super) fn use_def_map(&self, scope_id: FileScopeId) -> Arc<UseDefMap<'_>> {
         self.use_def_maps[scope_id].clone()
     }
 
@@ -299,28 +301,24 @@ impl<'db> SemanticIndex<'db> {
         &self.scopes[id]
     }
 
-    pub(crate) fn scope_ids(&self) -> impl Iterator<Item = ScopeId> {
+    pub(crate) fn scope_ids(&self) -> impl Iterator<Item = ScopeId<'db>> + '_ {
         self.scope_ids_by_scope.iter().copied()
     }
 
     pub(crate) fn symbol_is_global_in_scope(
         &self,
-        symbol: ScopedPlaceId,
+        symbol: ScopedSymbolId,
         scope: FileScopeId,
     ) -> bool {
-        self.place_table(scope)
-            .place_expr(symbol)
-            .is_marked_global()
+        self.place_table(scope).symbol(symbol).is_global()
     }
 
     pub(crate) fn symbol_is_nonlocal_in_scope(
         &self,
-        symbol: ScopedPlaceId,
+        symbol: ScopedSymbolId,
         scope: FileScopeId,
     ) -> bool {
-        self.place_table(scope)
-            .place_expr(symbol)
-            .is_marked_nonlocal()
+        self.place_table(scope).symbol(symbol).is_nonlocal()
     }
 
     /// Returns the id of the parent scope.
@@ -373,18 +371,18 @@ impl<'db> SemanticIndex<'db> {
 
     /// Returns an iterator over the descendent scopes of `scope`.
     #[allow(unused)]
-    pub(crate) fn descendent_scopes(&self, scope: FileScopeId) -> DescendantsIter {
+    pub(crate) fn descendent_scopes(&self, scope: FileScopeId) -> DescendantsIter<'_> {
         DescendantsIter::new(self, scope)
     }
 
     /// Returns an iterator over the direct child scopes of `scope`.
     #[allow(unused)]
-    pub(crate) fn child_scopes(&self, scope: FileScopeId) -> ChildrenIter {
+    pub(crate) fn child_scopes(&self, scope: FileScopeId) -> ChildrenIter<'_> {
         ChildrenIter::new(self, scope)
     }
 
     /// Returns an iterator over all ancestors of `scope`, starting with `scope` itself.
-    pub(crate) fn ancestor_scopes(&self, scope: FileScopeId) -> AncestorsIter {
+    pub(crate) fn ancestor_scopes(&self, scope: FileScopeId) -> AncestorsIter<'_> {
         AncestorsIter::new(self, scope)
     }
 
@@ -402,7 +400,7 @@ impl<'db> SemanticIndex<'db> {
     ///         print(x)  # Refers to global x=1, not class x=2
     /// ```
     /// The `method` function can see the global scope but not the class scope.
-    pub(crate) fn visible_ancestor_scopes(&self, scope: FileScopeId) -> VisibleAncestorsIter {
+    pub(crate) fn visible_ancestor_scopes(&self, scope: FileScopeId) -> VisibleAncestorsIter<'_> {
         VisibleAncestorsIter::new(self, scope)
     }
 
@@ -484,36 +482,53 @@ impl<'db> SemanticIndex<'db> {
 
     /// Returns
     /// * `NoLongerInEagerContext` if the nested scope is no longer in an eager context
-    ///   (that is, not every scope that will be traversed is eager).
-    /// *  an iterator of bindings for a particular nested eager scope reference if the bindings exist.
-    /// *  a narrowing constraint if there are no bindings, but there is a narrowing constraint for an outer scope symbol.
-    /// * `NotFound` if the narrowing constraint / bindings do not exist in the nested eager scope.
-    pub(crate) fn eager_snapshot(
+    ///   (that is, not every scope that will be traversed is eager) and no lazy snapshots were found.
+    /// *  an iterator of bindings for a particular nested scope reference if the bindings exist.
+    /// *  a narrowing constraint if there are no bindings, but there is a narrowing constraint for an enclosing scope place.
+    /// * `NotFound` if the narrowing constraint / bindings do not exist in the nested scope.
+    pub(crate) fn enclosing_snapshot(
         &self,
         enclosing_scope: FileScopeId,
-        expr: &PlaceExpr,
+        expr: PlaceExprRef,
         nested_scope: FileScopeId,
-    ) -> EagerSnapshotResult<'_, 'db> {
+    ) -> EnclosingSnapshotResult<'_, 'db> {
         for (ancestor_scope_id, ancestor_scope) in self.ancestor_scopes(nested_scope) {
             if ancestor_scope_id == enclosing_scope {
                 break;
             }
             if !ancestor_scope.is_eager() {
-                return EagerSnapshotResult::NoLongerInEagerContext;
+                if let PlaceExprRef::Symbol(symbol) = expr {
+                    if let Some(place_id) =
+                        self.place_tables[enclosing_scope].symbol_id(symbol.name())
+                    {
+                        let key = EnclosingSnapshotKey {
+                            enclosing_scope,
+                            enclosing_place: place_id.into(),
+                            nested_scope,
+                            nested_laziness: ScopeLaziness::Lazy,
+                        };
+                        if let Some(id) = self.enclosing_snapshots.get(&key) {
+                            return self.use_def_maps[enclosing_scope]
+                                .enclosing_snapshot(*id, key.nested_laziness);
+                        }
+                    }
+                }
+                return EnclosingSnapshotResult::NoLongerInEagerContext;
             }
         }
-        let Some(place_id) = self.place_tables[enclosing_scope].place_id_by_expr(expr) else {
-            return EagerSnapshotResult::NotFound;
+        let Some(place_id) = self.place_tables[enclosing_scope].place_id(expr) else {
+            return EnclosingSnapshotResult::NotFound;
         };
-        let key = EagerSnapshotKey {
+        let key = EnclosingSnapshotKey {
             enclosing_scope,
             enclosing_place: place_id,
             nested_scope,
+            nested_laziness: ScopeLaziness::Eager,
         };
-        let Some(id) = self.eager_snapshots.get(&key) else {
-            return EagerSnapshotResult::NotFound;
+        let Some(id) = self.enclosing_snapshots.get(&key) else {
+            return EnclosingSnapshotResult::NotFound;
         };
-        self.use_def_maps[enclosing_scope].inner.eager_snapshot(*id)
+        self.use_def_maps[enclosing_scope].enclosing_snapshot(*id, key.nested_laziness)
     }
 
     pub(crate) fn semantic_syntax_errors(&self) -> &[SemanticSyntaxError] {
@@ -521,29 +536,7 @@ impl<'db> SemanticIndex<'db> {
     }
 }
 
-#[derive(Debug, PartialEq, Eq, Clone, salsa::Update, get_size2::GetSize)]
-pub(crate) struct ArcUseDefMap<'db> {
-    #[get_size(size_fn = untracked_arc_size)]
-    inner: Arc<UseDefMap<'db>>,
-}
-
-impl<'db> std::ops::Deref for ArcUseDefMap<'db> {
-    type Target = UseDefMap<'db>;
-
-    fn deref(&self) -> &Self::Target {
-        &self.inner
-    }
-}
-
-impl<'db> ArcUseDefMap<'db> {
-    pub(crate) fn new(inner: UseDefMap<'db>) -> Self {
-        Self {
-            inner: Arc::new(inner),
-        }
-    }
-}
-
-pub struct AncestorsIter<'a> {
+pub(crate) struct AncestorsIter<'a> {
     scopes: &'a IndexSlice<FileScopeId, Scope>,
     next_id: Option<FileScopeId>,
 }
@@ -571,7 +564,7 @@ impl<'a> Iterator for AncestorsIter<'a> {
 
 impl FusedIterator for AncestorsIter<'_> {}
 
-pub struct VisibleAncestorsIter<'a> {
+pub(crate) struct VisibleAncestorsIter<'a> {
     inner: AncestorsIter<'a>,
     starting_scope_kind: ScopeKind,
     yielded_count: usize,
@@ -618,7 +611,7 @@ impl<'a> Iterator for VisibleAncestorsIter<'a> {
 
 impl FusedIterator for VisibleAncestorsIter<'_> {}
 
-pub struct DescendantsIter<'a> {
+pub(crate) struct DescendantsIter<'a> {
     next_id: FileScopeId,
     descendants: std::slice::Iter<'a, Scope>,
 }
@@ -655,7 +648,7 @@ impl FusedIterator for DescendantsIter<'_> {}
 
 impl ExactSizeIterator for DescendantsIter<'_> {}
 
-pub struct ChildrenIter<'a> {
+pub(crate) struct ChildrenIter<'a> {
     parent: FileScopeId,
     descendants: DescendantsIter<'a>,
 }
@@ -725,13 +718,15 @@ mod tests {
     use crate::db::tests::{TestDb, TestDbBuilder};
     use crate::semantic_index::ast_ids::{HasScopedUseId, ScopedUseId};
     use crate::semantic_index::definition::{Definition, DefinitionKind};
-    use crate::semantic_index::place::{FileScopeId, PlaceTable, Scope, ScopeKind, ScopedPlaceId};
+    use crate::semantic_index::place::PlaceTable;
+    use crate::semantic_index::scope::{FileScopeId, Scope, ScopeKind};
+    use crate::semantic_index::symbol::ScopedSymbolId;
     use crate::semantic_index::use_def::UseDefMap;
     use crate::semantic_index::{global_scope, place_table, semantic_index, use_def_map};
 
     impl UseDefMap<'_> {
-        fn first_public_binding(&self, symbol: ScopedPlaceId) -> Option<Definition<'_>> {
-            self.end_of_scope_bindings(symbol)
+        fn first_public_binding(&self, symbol: ScopedSymbolId) -> Option<Definition<'_>> {
+            self.end_of_scope_symbol_bindings(symbol)
                 .find_map(|constrained_binding| constrained_binding.binding.definition())
         }
 
@@ -761,8 +756,8 @@ mod tests {
 
     fn names(table: &PlaceTable) -> Vec<String> {
         table
-            .places()
-            .filter_map(|expr| Some(expr.as_name()?.to_string()))
+            .symbols()
+            .map(|expr| expr.name().to_string())
             .collect()
     }
 
@@ -800,7 +795,7 @@ mod tests {
         let global_table = place_table(&db, scope);
 
         assert_eq!(names(global_table), vec!["foo"]);
-        let foo = global_table.place_id_by_name("foo").unwrap();
+        let foo = global_table.symbol_id("foo").unwrap();
 
         let use_def = use_def_map(&db, scope);
         let binding = use_def.first_public_binding(foo).unwrap();
@@ -832,18 +827,14 @@ mod tests {
         assert_eq!(names(global_table), vec!["foo"]);
         assert!(
             global_table
-                .place_by_name("foo")
+                .symbol_by_name("foo")
                 .is_some_and(|symbol| { symbol.is_bound() && !symbol.is_used() }),
             "symbols that are defined get the defined flag"
         );
 
         let use_def = use_def_map(&db, scope);
         let binding = use_def
-            .first_public_binding(
-                global_table
-                    .place_id_by_name("foo")
-                    .expect("symbol to exist"),
-            )
+            .first_public_binding(global_table.symbol_id("foo").expect("symbol to exist"))
             .unwrap();
         assert!(matches!(binding.kind(&db), DefinitionKind::ImportFrom(_)));
     }
@@ -857,13 +848,13 @@ mod tests {
         assert_eq!(names(global_table), vec!["foo", "x"]);
         assert!(
             global_table
-                .place_by_name("foo")
+                .symbol_by_name("foo")
                 .is_some_and(|symbol| { !symbol.is_bound() && symbol.is_used() }),
             "a symbol used but not bound in a scope should have only the used flag"
         );
         let use_def = use_def_map(&db, scope);
         let binding = use_def
-            .first_public_binding(global_table.place_id_by_name("x").expect("symbol exists"))
+            .first_public_binding(global_table.symbol_id("x").expect("symbol exists"))
             .unwrap();
         assert!(matches!(binding.kind(&db), DefinitionKind::Assignment(_)));
     }
@@ -878,7 +869,7 @@ mod tests {
 
         let use_def = use_def_map(&db, scope);
         let binding = use_def
-            .first_public_binding(global_table.place_id_by_name("x").unwrap())
+            .first_public_binding(global_table.symbol_id("x").unwrap())
             .unwrap();
 
         assert!(matches!(
@@ -920,7 +911,7 @@ y = 2
 
         let use_def = index.use_def_map(class_scope_id);
         let binding = use_def
-            .first_public_binding(class_table.place_id_by_name("x").expect("symbol exists"))
+            .first_public_binding(class_table.symbol_id("x").expect("symbol exists"))
             .unwrap();
         assert!(matches!(binding.kind(&db), DefinitionKind::Assignment(_)));
     }
@@ -957,7 +948,7 @@ y = 2
 
         let use_def = index.use_def_map(function_scope_id);
         let binding = use_def
-            .first_public_binding(function_table.place_id_by_name("x").expect("symbol exists"))
+            .first_public_binding(function_table.symbol_id("x").expect("symbol exists"))
             .unwrap();
         assert!(matches!(binding.kind(&db), DefinitionKind::Assignment(_)));
     }
@@ -992,31 +983,19 @@ def f(a: str, /, b: str, c: int = 1, *args, d: int = 2, **kwargs):
         let use_def = index.use_def_map(function_scope_id);
         for name in ["a", "b", "c", "d"] {
             let binding = use_def
-                .first_public_binding(
-                    function_table
-                        .place_id_by_name(name)
-                        .expect("symbol exists"),
-                )
+                .first_public_binding(function_table.symbol_id(name).expect("symbol exists"))
                 .unwrap();
             assert!(matches!(binding.kind(&db), DefinitionKind::Parameter(_)));
         }
         let args_binding = use_def
-            .first_public_binding(
-                function_table
-                    .place_id_by_name("args")
-                    .expect("symbol exists"),
-            )
+            .first_public_binding(function_table.symbol_id("args").expect("symbol exists"))
             .unwrap();
         assert!(matches!(
             args_binding.kind(&db),
             DefinitionKind::VariadicPositionalParameter(_)
         ));
         let kwargs_binding = use_def
-            .first_public_binding(
-                function_table
-                    .place_id_by_name("kwargs")
-                    .expect("symbol exists"),
-            )
+            .first_public_binding(function_table.symbol_id("kwargs").expect("symbol exists"))
             .unwrap();
         assert!(matches!(
             kwargs_binding.kind(&db),
@@ -1049,27 +1028,19 @@ def f(a: str, /, b: str, c: int = 1, *args, d: int = 2, **kwargs):
         let use_def = index.use_def_map(lambda_scope_id);
         for name in ["a", "b", "c", "d"] {
             let binding = use_def
-                .first_public_binding(lambda_table.place_id_by_name(name).expect("symbol exists"))
+                .first_public_binding(lambda_table.symbol_id(name).expect("symbol exists"))
                 .unwrap();
             assert!(matches!(binding.kind(&db), DefinitionKind::Parameter(_)));
         }
         let args_binding = use_def
-            .first_public_binding(
-                lambda_table
-                    .place_id_by_name("args")
-                    .expect("symbol exists"),
-            )
+            .first_public_binding(lambda_table.symbol_id("args").expect("symbol exists"))
             .unwrap();
         assert!(matches!(
             args_binding.kind(&db),
             DefinitionKind::VariadicPositionalParameter(_)
         ));
         let kwargs_binding = use_def
-            .first_public_binding(
-                lambda_table
-                    .place_id_by_name("kwargs")
-                    .expect("symbol exists"),
-            )
+            .first_public_binding(lambda_table.symbol_id("kwargs").expect("symbol exists"))
             .unwrap();
         assert!(matches!(
             kwargs_binding.kind(&db),
@@ -1117,7 +1088,7 @@ def f(a: str, /, b: str, c: int = 1, *args, d: int = 2, **kwargs):
             let binding = use_def
                 .first_public_binding(
                     comprehension_symbol_table
-                        .place_id_by_name(name)
+                        .symbol_id(name)
                         .expect("symbol exists"),
                 )
                 .unwrap();
@@ -1246,7 +1217,7 @@ with item1 as x, item2 as y:
         let use_def = index.use_def_map(FileScopeId::global());
         for name in ["x", "y"] {
             let binding = use_def
-                .first_public_binding(global_table.place_id_by_name(name).expect("symbol exists"))
+                .first_public_binding(global_table.symbol_id(name).expect("symbol exists"))
                 .expect("Expected with item definition for {name}");
             assert!(matches!(binding.kind(&db), DefinitionKind::WithItem(_)));
         }
@@ -1269,7 +1240,7 @@ with context() as (x, y):
         let use_def = index.use_def_map(FileScopeId::global());
         for name in ["x", "y"] {
             let binding = use_def
-                .first_public_binding(global_table.place_id_by_name(name).expect("symbol exists"))
+                .first_public_binding(global_table.symbol_id(name).expect("symbol exists"))
                 .expect("Expected with item definition for {name}");
             assert!(matches!(binding.kind(&db), DefinitionKind::WithItem(_)));
         }
@@ -1319,11 +1290,7 @@ def func():
 
         let use_def = index.use_def_map(FileScopeId::global());
         let binding = use_def
-            .first_public_binding(
-                global_table
-                    .place_id_by_name("func")
-                    .expect("symbol exists"),
-            )
+            .first_public_binding(global_table.symbol_id("func").expect("symbol exists"))
             .unwrap();
         assert!(matches!(binding.kind(&db), DefinitionKind::Function(_)));
     }
@@ -1400,7 +1367,7 @@ class C[T]:
         assert_eq!(names(&ann_table), vec!["T"]);
         assert!(
             ann_table
-                .place_by_name("T")
+                .symbol_by_name("T")
                 .is_some_and(|s| s.is_bound() && !s.is_used()),
             "type parameters are defined by the scope that introduces them"
         );
@@ -1548,7 +1515,7 @@ match subject:
         let global_scope_id = global_scope(&db, file);
         let global_table = place_table(&db, global_scope_id);
 
-        assert!(global_table.place_by_name("Foo").unwrap().is_used());
+        assert!(global_table.symbol_by_name("Foo").unwrap().is_used());
         assert_eq!(
             names(global_table),
             vec![
@@ -1572,7 +1539,7 @@ match subject:
             ("l", 1),
         ] {
             let binding = use_def
-                .first_public_binding(global_table.place_id_by_name(name).expect("symbol exists"))
+                .first_public_binding(global_table.symbol_id(name).expect("symbol exists"))
                 .expect("Expected with item definition for {name}");
             if let DefinitionKind::MatchPattern(pattern) = binding.kind(&db) {
                 assert_eq!(pattern.index(), expected_index);
@@ -1602,7 +1569,7 @@ match 1:
         let use_def = use_def_map(&db, global_scope_id);
         for (name, expected_index) in [("first", 0), ("second", 0)] {
             let binding = use_def
-                .first_public_binding(global_table.place_id_by_name(name).expect("symbol exists"))
+                .first_public_binding(global_table.symbol_id(name).expect("symbol exists"))
                 .expect("Expected with item definition for {name}");
             if let DefinitionKind::MatchPattern(pattern) = binding.kind(&db) {
                 assert_eq!(pattern.index(), expected_index);
@@ -1622,7 +1589,7 @@ match 1:
 
         let use_def = use_def_map(&db, scope);
         let binding = use_def
-            .first_public_binding(global_table.place_id_by_name("x").unwrap())
+            .first_public_binding(global_table.symbol_id("x").unwrap())
             .unwrap();
 
         assert!(matches!(binding.kind(&db), DefinitionKind::For(_)));
@@ -1638,10 +1605,10 @@ match 1:
 
         let use_def = use_def_map(&db, scope);
         let x_binding = use_def
-            .first_public_binding(global_table.place_id_by_name("x").unwrap())
+            .first_public_binding(global_table.symbol_id("x").unwrap())
             .unwrap();
         let y_binding = use_def
-            .first_public_binding(global_table.place_id_by_name("y").unwrap())
+            .first_public_binding(global_table.symbol_id("y").unwrap())
             .unwrap();
 
         assert!(matches!(x_binding.kind(&db), DefinitionKind::For(_)));
@@ -1658,7 +1625,7 @@ match 1:
 
         let use_def = use_def_map(&db, scope);
         let binding = use_def
-            .first_public_binding(global_table.place_id_by_name("a").unwrap())
+            .first_public_binding(global_table.symbol_id("a").unwrap())
             .unwrap();
 
         assert!(matches!(binding.kind(&db), DefinitionKind::For(_)));
