@@ -1,37 +1,45 @@
 //! Data model, state management, and configuration resolution.
 
-use std::collections::{BTreeMap, VecDeque};
-use std::ops::{Deref, DerefMut};
-use std::panic::RefUnwindSafe;
-use std::sync::Arc;
-
 use anyhow::{Context, anyhow};
 use index::DocumentQueryError;
-use lsp_server::Message;
+use lsp_server::{Message, RequestId};
 use lsp_types::notification::{Exit, Notification};
-use lsp_types::request::{Request, Shutdown};
-use lsp_types::{ClientCapabilities, TextDocumentContentChangeEvent, Url};
+use lsp_types::request::{
+    DocumentDiagnosticRequest, RegisterCapability, Rename, Request, Shutdown, UnregisterCapability,
+    WorkspaceDiagnosticRequest,
+};
+use lsp_types::{
+    DiagnosticRegistrationOptions, DiagnosticServerCapabilities, Registration, RegistrationParams,
+    TextDocumentContentChangeEvent, Unregistration, UnregistrationParams, Url,
+};
 use options::GlobalOptions;
 use ruff_db::Db;
 use ruff_db::files::File;
 use ruff_db::system::{System, SystemPath, SystemPathBuf};
+use std::collections::{BTreeMap, HashSet, VecDeque};
+use std::ops::{Deref, DerefMut};
+use std::panic::RefUnwindSafe;
+use std::sync::Arc;
+use ty_combine::Combine;
 use ty_project::metadata::Options;
 use ty_project::watch::ChangeEvent;
-use ty_project::{ChangeResult, Db as _, ProjectDatabase, ProjectMetadata};
+use ty_project::{ChangeResult, CheckMode, Db as _, ProjectDatabase, ProjectMetadata};
 
-pub(crate) use self::capabilities::ResolvedClientCapabilities;
 pub(crate) use self::index::DocumentQuery;
-pub(crate) use self::options::{AllOptions, ClientOptions, DiagnosticMode};
-pub(crate) use self::settings::ClientSettings;
+pub(crate) use self::options::InitializationOptions;
+pub use self::options::{ClientOptions, DiagnosticMode};
+pub(crate) use self::settings::{GlobalSettings, WorkspaceSettings};
+use crate::capabilities::{
+    ResolvedClientCapabilities, server_diagnostic_options, server_rename_options,
+};
 use crate::document::{DocumentKey, DocumentVersion, NotebookDocument};
-use crate::server::publish_settings_diagnostics;
+use crate::server::{Action, publish_settings_diagnostics};
 use crate::session::client::Client;
 use crate::session::request_queue::RequestQueue;
 use crate::system::{AnySystemPath, LSPSystem};
 use crate::{PositionEncoding, TextDocument};
 use index::Index;
 
-mod capabilities;
 pub(crate) mod client;
 pub(crate) mod index;
 mod options;
@@ -65,6 +73,12 @@ pub(crate) struct Session {
     /// That's what we use the default project for.
     default_project: DefaultProject,
 
+    /// Initialization options that were provided by the client during server initialization.
+    initialization_options: InitializationOptions,
+
+    /// Resolved global settings that are shared across all workspaces.
+    global_settings: Arc<GlobalSettings>,
+
     /// The global position encoding, negotiated during LSP initialization.
     position_encoding: PositionEncoding,
 
@@ -77,12 +91,29 @@ pub(crate) struct Session {
     /// Has the client requested the server to shutdown.
     shutdown_requested: bool,
 
+    /// Whether the server has dynamically registered the diagnostic capability with the client.
+    /// Is the connected client a `TestServer` instance.
+    in_test: bool,
+
     deferred_messages: VecDeque<Message>,
+
+    /// A revision counter. It gets incremented on every change to `Session` that
+    /// could result in different workspace diagnostics.
+    revision: u64,
+
+    /// A pending workspace diagnostics request because there were no diagnostics
+    /// or no changes when when the request ran last time.
+    /// We'll re-run the request after every change to `Session` (see `revision`)
+    /// to see if there are now changes and, if so, respond to the client.
+    suspended_workspace_diagnostics_request: Option<SuspendedWorkspaceDiagnosticRequest>,
+
+    /// Registrations is a set of LSP methods that have been dynamically registered with the
+    /// client.
+    registrations: HashSet<String>,
 }
 
 /// LSP State for a Project
 pub(crate) struct ProjectState {
-    pub(crate) db: ProjectDatabase,
     /// Files that we have outstanding otherwise-untracked pushed diagnostics for.
     ///
     /// In `CheckMode::OpenFiles` we still read some files that the client hasn't
@@ -97,21 +128,32 @@ pub(crate) struct ProjectState {
     /// for so that we can clear the diagnostics for all of them before we go
     /// to update any of them.
     pub(crate) untracked_files_with_pushed_diagnostics: Vec<Url>,
+
+    // Note: This field should be last to ensure the `db` gets dropped last.
+    // The db drop order matters because we call `Arc::into_inner` on some Arc's
+    // and we use Salsa's cancellation to guarantee that there's only a single reference to the `Arc`.
+    // However, this requires that the db drops last.
+    // This shouldn't matter here because the db's stored in the session are the
+    // only reference we want to hold on, but better be safe than sorry ;).
+    pub(crate) db: ProjectDatabase,
 }
 
 impl Session {
     pub(crate) fn new(
-        client_capabilities: &ClientCapabilities,
+        resolved_client_capabilities: ResolvedClientCapabilities,
         position_encoding: PositionEncoding,
-        global_options: GlobalOptions,
-        workspace_folders: Vec<(Url, ClientOptions)>,
+        workspace_urls: Vec<Url>,
+        initialization_options: InitializationOptions,
         native_system: Arc<dyn System + 'static + Send + Sync + RefUnwindSafe>,
+        in_test: bool,
     ) -> crate::Result<Self> {
-        let index = Arc::new(Index::new(global_options.into_settings()));
+        let index = Arc::new(Index::new());
 
         let mut workspaces = Workspaces::default();
-        for (url, workspace_options) in workspace_folders {
-            workspaces.register(url, workspace_options.into_settings())?;
+        // Register workspaces with default settings - they'll be initialized with real settings
+        // when workspace/configuration response is received
+        for url in workspace_urls {
+            workspaces.register(url)?;
         }
 
         Ok(Self {
@@ -121,20 +163,31 @@ impl Session {
             deferred_messages: VecDeque::new(),
             index: Some(index),
             default_project: DefaultProject::new(),
+            initialization_options,
+            global_settings: Arc::new(GlobalSettings::default()),
             projects: BTreeMap::new(),
-            resolved_client_capabilities: ResolvedClientCapabilities::new(client_capabilities),
+            resolved_client_capabilities,
             request_queue: RequestQueue::new(),
             shutdown_requested: false,
+            in_test,
+            suspended_workspace_diagnostics_request: None,
+            revision: 0,
+            registrations: HashSet::new(),
         })
     }
 
     pub(crate) fn request_queue(&self) -> &RequestQueue {
         &self.request_queue
     }
+
     pub(crate) fn request_queue_mut(&mut self) -> &mut RequestQueue {
         &mut self.request_queue
     }
 
+    pub(crate) fn initialization_options(&self) -> &InitializationOptions {
+        &self.initialization_options
+    }
+
     pub(crate) fn is_shutdown_requested(&self) -> bool {
         self.shutdown_requested
     }
@@ -143,6 +196,56 @@ impl Session {
         self.shutdown_requested = requested;
     }
 
+    pub(crate) fn set_suspended_workspace_diagnostics_request(
+        &mut self,
+        request: SuspendedWorkspaceDiagnosticRequest,
+        client: &Client,
+    ) {
+        self.suspended_workspace_diagnostics_request = Some(request);
+        // Run the suspended workspace diagnostic request immediately in case there
+        // were changes since the workspace diagnostics background thread queued
+        // the action to suspend the workspace diagnostic request.
+        self.resume_suspended_workspace_diagnostic_request(client);
+    }
+
+    pub(crate) fn take_suspended_workspace_diagnostic_request(
+        &mut self,
+    ) -> Option<SuspendedWorkspaceDiagnosticRequest> {
+        self.suspended_workspace_diagnostics_request.take()
+    }
+
+    /// Resumes (retries) the workspace diagnostic request if there
+    /// were any changes to the [`Session`] (the revision got bumped)
+    /// since the workspace diagnostic request ran last time.
+    ///
+    /// The workspace diagnostic requests is ignored if the request
+    /// was cancelled in the meantime.
+    pub(crate) fn resume_suspended_workspace_diagnostic_request(&mut self, client: &Client) {
+        self.suspended_workspace_diagnostics_request = self
+            .suspended_workspace_diagnostics_request
+            .take()
+            .and_then(|request| {
+                if !self.request_queue.incoming().is_pending(&request.id) {
+                    // Clear out the suspended request if the request has been cancelled.
+                    tracing::debug!("Skipping suspended workspace diagnostics request `{}` because it was cancelled", request.id);
+                    return None;
+                }
+
+                request.resume_if_revision_changed(self.revision, client)
+            });
+    }
+
+    /// Bumps the revision.
+    ///
+    /// The revision is used to track when workspace diagnostics may have changed and need to be re-run.
+    /// It's okay if a bump doesn't necessarily result in new workspace diagnostics.
+    ///
+    /// In general, any change to a project database should bump the revision and so should
+    /// any change to the document states (but also when the open workspaces change etc.).
+    fn bump_revision(&mut self) {
+        self.revision += 1;
+    }
+
     /// The LSP specification doesn't allow configuration requests during initialization,
     /// but we need access to the configuration to resolve the settings in turn to create the
     /// project databases. This will become more important in the future when we support
@@ -306,6 +409,8 @@ impl Session {
                 .cloned()
         });
 
+        self.bump_revision();
+
         self.project_db_mut(path)
             .apply_changes(changes, overrides.as_ref())
     }
@@ -339,11 +444,54 @@ impl Session {
     ) {
         assert!(!self.workspaces.all_initialized());
 
+        // These are the options combined from all the global options received by the server for
+        // each workspace via the workspace configuration request.
+        let mut combined_global_options: Option<GlobalOptions> = None;
+
         for (url, options) in workspace_settings {
             tracing::debug!("Initializing workspace `{url}`");
 
-            let settings = options.into_settings();
-            let Some((root, workspace)) = self.workspaces.initialize(&url, settings) else {
+            // Combine the global options specified during initialization with the
+            // workspace-specific options to create the final workspace options.
+            let ClientOptions {
+                global, workspace, ..
+            } = self
+                .initialization_options
+                .options
+                .clone()
+                .combine(options.clone());
+
+            let unknown_options = &options.unknown;
+            if !unknown_options.is_empty() {
+                // HACK: This is to ensure that users with an older version of the ty VS Code
+                // extension don't get warnings about unknown options when they are using a newer
+                // version of the language server. This should be removed after a few releases.
+                if !unknown_options.contains_key("importStrategy")
+                    && !unknown_options.contains_key("interpreter")
+                {
+                    tracing::warn!(
+                        "Received unknown options for workspace `{url}`: {}",
+                        serde_json::to_string_pretty(unknown_options)
+                            .unwrap_or_else(|_| format!("{unknown_options:?}"))
+                    );
+
+                    client.show_warning_message(format!(
+                        "Received unknown options for workspace `{url}`: '{}'. \
+                        Refer to the logs for more details.",
+                        unknown_options
+                            .keys()
+                            .map(String::as_str)
+                            .collect::<Vec<_>>()
+                            .join("', '")
+                    ));
+                }
+            }
+
+            combined_global_options.combine_with(Some(global));
+
+            let workspace_settings = workspace.into_settings();
+            let Some((root, workspace)) = self.workspaces.initialize(&url, workspace_settings)
+            else {
                 continue;
             };
 
@@ -370,17 +518,16 @@ impl Session {
                 });
 
             let (root, db) = match project {
-                Ok(mut db) => {
-                    db.set_check_mode(workspace.settings.diagnostic_mode().into_check_mode());
-                    (root, db)
-                }
+                Ok(db) => (root, db),
                 Err(err) => {
                     tracing::error!(
-                        "Failed to create project for `{root}`: {err:#}. Falling back to default settings"
+                        "Failed to create project for `{root}`: {err:#}. \
+                        Falling back to default settings"
                     );
 
                     client.show_error_message(format!(
-                        "Failed to load project rooted at {root}. Please refer to the logs for more details.",
+                        "Failed to load project rooted at {root}. \
+                        Please refer to the logs for more details.",
                     ));
 
                     let db_with_default_settings =
@@ -413,6 +560,19 @@ impl Session {
             publish_settings_diagnostics(self, client, root);
         }
 
+        if let Some(global_options) = combined_global_options.take() {
+            let global_settings = global_options.into_settings();
+            if global_settings.diagnostic_mode().is_workspace() {
+                for project in self.projects.values_mut() {
+                    project.db.set_check_mode(CheckMode::AllFiles);
+                }
+            }
+            self.global_settings = Arc::new(global_settings);
+        }
+
+        self.register_diagnostic_capability(client);
+        self.register_rename_capability(client);
+
         assert!(
             self.workspaces.all_initialized(),
             "All workspaces should be initialized after calling `initialize_workspaces`"
@@ -427,17 +587,150 @@ impl Session {
         }
     }
 
+    // TODO: Merge the following two methods as `register_capability` and `unregister_capability`
+
+    /// Sends a registration notification to the client to enable / disable workspace diagnostics
+    /// as per the `ty.diagnosticMode` global setting.
+    ///
+    /// This method is a no-op if the client doesn't support dynamic registration of diagnostic
+    /// capability.
+    fn register_diagnostic_capability(&mut self, client: &Client) {
+        static DIAGNOSTIC_REGISTRATION_ID: &str = "ty/textDocument/diagnostic";
+
+        if !self
+            .resolved_client_capabilities
+            .supports_diagnostic_dynamic_registration()
+        {
+            return;
+        }
+
+        let registered = self
+            .registrations
+            .contains(DocumentDiagnosticRequest::METHOD);
+
+        if registered {
+            client.send_request::<UnregisterCapability>(
+                self,
+                UnregistrationParams {
+                    unregisterations: vec![Unregistration {
+                        id: DIAGNOSTIC_REGISTRATION_ID.into(),
+                        method: DocumentDiagnosticRequest::METHOD.into(),
+                    }],
+                },
+                |_: &Client, ()| {
+                    tracing::debug!("Unregistered diagnostic capability");
+                },
+            );
+        }
+
+        let diagnostic_mode = self.global_settings.diagnostic_mode;
+
+        let registration = Registration {
+            id: DIAGNOSTIC_REGISTRATION_ID.into(),
+            method: DocumentDiagnosticRequest::METHOD.into(),
+            register_options: Some(
+                serde_json::to_value(DiagnosticServerCapabilities::RegistrationOptions(
+                    DiagnosticRegistrationOptions {
+                        diagnostic_options: server_diagnostic_options(
+                            diagnostic_mode.is_workspace(),
+                        ),
+                        ..Default::default()
+                    },
+                ))
+                .unwrap(),
+            ),
+        };
+
+        client.send_request::<RegisterCapability>(
+            self,
+            RegistrationParams {
+                registrations: vec![registration],
+            },
+            move |_: &Client, ()| {
+                tracing::debug!(
+                    "Registered diagnostic capability in {diagnostic_mode:?} diagnostic mode"
+                );
+            },
+        );
+
+        if !registered {
+            self.registrations
+                .insert(DocumentDiagnosticRequest::METHOD.to_string());
+        }
+    }
+
+    /// Sends a registration notification to the client to enable / disable rename capability as
+    /// per the `ty.experimental.rename` global setting.
+    ///
+    /// This method is a no-op if the client doesn't support dynamic registration of rename
+    /// capability.
+    fn register_rename_capability(&mut self, client: &Client) {
+        static RENAME_REGISTRATION_ID: &str = "ty/textDocument/rename";
+
+        if !self
+            .resolved_client_capabilities
+            .supports_rename_dynamic_registration()
+        {
+            return;
+        }
+
+        let registered = self.registrations.contains(Rename::METHOD);
+
+        if registered {
+            client.send_request::<UnregisterCapability>(
+                self,
+                UnregistrationParams {
+                    unregisterations: vec![Unregistration {
+                        id: RENAME_REGISTRATION_ID.into(),
+                        method: Rename::METHOD.into(),
+                    }],
+                },
+                move |_: &Client, ()| {
+                    tracing::debug!("Unregistered rename capability");
+                },
+            );
+        }
+
+        if !self.global_settings.experimental.rename {
+            tracing::debug!("Rename capability is disabled in the client settings");
+            return;
+        }
+
+        let registration = Registration {
+            id: RENAME_REGISTRATION_ID.into(),
+            method: Rename::METHOD.into(),
+            register_options: Some(serde_json::to_value(server_rename_options()).unwrap()),
+        };
+
+        client.send_request::<RegisterCapability>(
+            self,
+            RegistrationParams {
+                registrations: vec![registration],
+            },
+            move |_: &Client, ()| {
+                tracing::debug!("Registered rename capability");
+            },
+        );
+
+        if !registered {
+            self.registrations.insert(Rename::METHOD.to_string());
+        }
+    }
+
     /// Creates a document snapshot with the URL referencing the document to snapshot.
     pub(crate) fn take_document_snapshot(&self, url: Url) -> DocumentSnapshot {
-        let index = self.index();
+        let key = self
+            .key_from_url(url)
+            .map_err(DocumentQueryError::InvalidUrl);
         DocumentSnapshot {
             resolved_client_capabilities: self.resolved_client_capabilities,
-            client_settings: index.global_settings(),
+            workspace_settings: key
+                .as_ref()
+                .ok()
+                .and_then(|key| self.workspaces.settings_for_path(key.path().as_system()?))
+                .unwrap_or_else(|| Arc::new(WorkspaceSettings::default())),
             position_encoding: self.position_encoding,
-            document_query_result: self
-                .key_from_url(url)
-                .map_err(DocumentQueryError::InvalidUrl)
-                .and_then(|key| index.make_document_ref(key)),
+            document_query_result: key.and_then(|key| self.index().make_document_ref(key)),
         }
     }
 
@@ -451,7 +744,11 @@ impl Session {
                 .cloned()
                 .collect(),
             index: self.index.clone().unwrap(),
+            global_settings: self.global_settings.clone(),
             position_encoding: self.position_encoding,
+            in_test: self.in_test,
+            resolved_client_capabilities: self.resolved_client_capabilities,
+            revision: self.revision,
         }
     }
 
@@ -470,12 +767,14 @@ impl Session {
         document: NotebookDocument,
     ) {
         self.index_mut().open_notebook_document(path, document);
+        self.bump_revision();
     }
 
     /// Registers a text document at the provided `path`.
     /// If a document is already open here, it will be overwritten.
     pub(crate) fn open_text_document(&mut self, path: &AnySystemPath, document: TextDocument) {
         self.index_mut().open_text_document(path, document);
+        self.bump_revision();
     }
 
     /// Updates a text document at the associated `key`.
@@ -488,14 +787,21 @@ impl Session {
         new_version: DocumentVersion,
     ) -> crate::Result<()> {
         let position_encoding = self.position_encoding;
-        self.index_mut()
-            .update_text_document(key, content_changes, new_version, position_encoding)
+        self.index_mut().update_text_document(
+            key,
+            content_changes,
+            new_version,
+            position_encoding,
+        )?;
+        self.bump_revision();
+        Ok(())
     }
 
     /// De-registers a document, specified by its key.
     /// Calling this multiple times for the same document is a logic error.
     pub(crate) fn close_document(&mut self, key: &DocumentKey) -> crate::Result<()> {
         self.index_mut().close_document(key)?;
+        self.bump_revision();
         Ok(())
     }
 
@@ -515,7 +821,7 @@ impl Session {
     /// This method drops all references to the index and returns a guard that will restore the
     /// references when dropped. This guard holds the only reference to the index and allows
     /// modifying it.
-    fn index_mut(&mut self) -> MutIndexGuard {
+    fn index_mut(&mut self) -> MutIndexGuard<'_> {
         let index = self.index.take().unwrap();
 
         for db in self.projects_mut() {
@@ -540,8 +846,8 @@ impl Session {
         self.resolved_client_capabilities
     }
 
-    pub(crate) fn global_settings(&self) -> Arc<ClientSettings> {
-        self.index().global_settings()
+    pub(crate) fn global_settings(&self) -> &GlobalSettings {
+        &self.global_settings
     }
 
     pub(crate) fn position_encoding(&self) -> PositionEncoding {
@@ -592,7 +898,7 @@ impl Drop for MutIndexGuard<'_> {
 #[derive(Debug)]
 pub(crate) struct DocumentSnapshot {
     resolved_client_capabilities: ResolvedClientCapabilities,
-    client_settings: Arc<ClientSettings>,
+    workspace_settings: Arc<WorkspaceSettings>,
     position_encoding: PositionEncoding,
     document_query_result: Result<DocumentQuery, DocumentQueryError>,
 }
@@ -608,9 +914,9 @@ impl DocumentSnapshot {
         self.position_encoding
     }
 
-    /// Returns the client settings for this document.
-    pub(crate) fn client_settings(&self) -> &ClientSettings {
-        &self.client_settings
+    /// Returns the client settings for the workspace that this document belongs to.
+    pub(crate) fn workspace_settings(&self) -> &WorkspaceSettings {
+        &self.workspace_settings
     }
 
     /// Returns the result of the document query for this snapshot.
@@ -639,9 +945,23 @@ impl DocumentSnapshot {
 
 /// An immutable snapshot of the current state of [`Session`].
 pub(crate) struct SessionSnapshot {
-    projects: Vec<ProjectDatabase>,
     index: Arc<Index>,
+    global_settings: Arc<GlobalSettings>,
     position_encoding: PositionEncoding,
+    resolved_client_capabilities: ResolvedClientCapabilities,
+    in_test: bool,
+    revision: u64,
+
+    /// IMPORTANT: It's important that the databases come last, or at least,
+    /// after any `Arc` that we try to extract or mutate in-place using `Arc::into_inner`
+    /// and that relies on Salsa's cancellation to guarantee that there's now only a
+    /// single reference to it (e.g. see [`Session::index_mut`]).
+    ///
+    /// Making this field come last guarantees that the db's `Drop` handler is
+    /// dropped after all other fields, which ensures that
+    /// Salsa's cancellation blocks until all fields are dropped (and not only
+    /// waits for the db to be dropped while we still hold on to the `Index`).
+    projects: Vec<ProjectDatabase>,
 }
 
 impl SessionSnapshot {
@@ -653,9 +973,25 @@ impl SessionSnapshot {
         &self.index
     }
 
+    pub(crate) fn global_settings(&self) -> &GlobalSettings {
+        &self.global_settings
+    }
+
     pub(crate) fn position_encoding(&self) -> PositionEncoding {
         self.position_encoding
     }
+
+    pub(crate) fn resolved_client_capabilities(&self) -> ResolvedClientCapabilities {
+        self.resolved_client_capabilities
+    }
+
+    pub(crate) const fn in_test(&self) -> bool {
+        self.in_test
+    }
+
+    pub(crate) fn revision(&self) -> u64 {
+        self.revision
+    }
 }
 
 #[derive(Debug, Default)]
@@ -672,7 +1008,9 @@ impl Workspaces {
     /// the workspace are announced to the server during the `initialize` request, but the
     /// resolved settings are only available after the client has responded to the `workspace/configuration`
     /// request.
-    pub(crate) fn register(&mut self, url: Url, settings: ClientSettings) -> anyhow::Result<()> {
+    ///
+    /// [`initialize`]: Workspaces::initialize
+    pub(crate) fn register(&mut self, url: Url) -> anyhow::Result<()> {
         let path = url
             .to_file_path()
             .map_err(|()| anyhow!("Workspace URL is not a file or directory: {url:?}"))?;
@@ -681,8 +1019,13 @@ impl Workspaces {
         let system_path = SystemPathBuf::from_path_buf(path)
             .map_err(|_| anyhow!("Workspace URL is not valid UTF8"))?;
 
-        self.workspaces
-            .insert(system_path, Workspace { url, settings });
+        self.workspaces.insert(
+            system_path,
+            Workspace {
+                url,
+                settings: Arc::new(WorkspaceSettings::default()),
+            },
+        );
 
         self.uninitialized += 1;
 
@@ -697,7 +1040,7 @@ impl Workspaces {
     pub(crate) fn initialize(
         &mut self,
         url: &Url,
-        settings: ClientSettings,
+        settings: WorkspaceSettings,
     ) -> Option<(SystemPathBuf, &mut Workspace)> {
         let path = url.to_file_path().ok()?;
 
@@ -705,7 +1048,7 @@ impl Workspaces {
         let system_path = SystemPathBuf::from_path_buf(path).ok()?;
 
         if let Some(workspace) = self.workspaces.get_mut(&system_path) {
-            workspace.settings = settings;
+            workspace.settings = Arc::new(settings);
             self.uninitialized -= 1;
             Some((system_path, workspace))
         } else {
@@ -713,6 +1056,8 @@ impl Workspaces {
         }
     }
 
+    /// Returns a reference to the workspace for the given path, [`None`] if there's no workspace
+    /// registered for the path.
     pub(crate) fn for_path(&self, path: impl AsRef<SystemPath>) -> Option<&Workspace> {
         self.workspaces
             .range(..=path.as_ref().to_path_buf())
@@ -720,10 +1065,22 @@ impl Workspaces {
             .map(|(_, db)| db)
     }
 
+    /// Returns the client settings for the workspace at the given path, [`None`] if there's no
+    /// workspace registered for the path.
+    pub(crate) fn settings_for_path(
+        &self,
+        path: impl AsRef<SystemPath>,
+    ) -> Option<Arc<WorkspaceSettings>> {
+        self.for_path(path).map(Workspace::settings_arc)
+    }
+
     pub(crate) fn urls(&self) -> impl Iterator<Item = &Url> + '_ {
         self.workspaces.values().map(Workspace::url)
     }
 
+    /// Returns `true` if all workspaces have been [initialized].
+    ///
+    /// [initialized]: Workspaces::initialize
     pub(crate) fn all_initialized(&self) -> bool {
         self.uninitialized == 0
     }
@@ -742,7 +1099,7 @@ impl<'a> IntoIterator for &'a Workspaces {
 pub(crate) struct Workspace {
     /// The workspace root URL as sent by the client during initialization.
     url: Url,
-    settings: ClientSettings,
+    settings: Arc<WorkspaceSettings>,
 }
 
 impl Workspace {
@@ -750,9 +1107,13 @@ impl Workspace {
         &self.url
     }
 
-    pub(crate) fn settings(&self) -> &ClientSettings {
+    pub(crate) fn settings(&self) -> &WorkspaceSettings {
         &self.settings
     }
+
+    pub(crate) fn settings_arc(&self) -> Arc<WorkspaceSettings> {
+        self.settings.clone()
+    }
 }
 
 /// Thin wrapper around the default project database that ensures it only gets initialized
@@ -788,11 +1149,8 @@ impl DefaultProject {
             )
             .unwrap();
 
-            let mut db = ProjectDatabase::new(metadata, system).unwrap();
-            db.set_check_mode(index.global_settings().diagnostic_mode().into_check_mode());
-
             ProjectState {
-                db,
+                db: ProjectDatabase::new(metadata, system).unwrap(),
                 untracked_files_with_pushed_diagnostics: Vec::new(),
             }
         })
@@ -814,3 +1172,43 @@ impl DefaultProject {
         self.0.get_mut()
     }
 }
+
+/// A workspace diagnostic request that didn't yield any changes or diagnostic
+/// when it ran the last time.
+#[derive(Debug)]
+pub(crate) struct SuspendedWorkspaceDiagnosticRequest {
+    /// The LSP request id
+    pub(crate) id: RequestId,
+
+    /// The params passed to the `workspace/diagnostic` request.
+    pub(crate) params: serde_json::Value,
+
+    /// The session's revision when the request ran the last time.
+    ///
+    /// This is to prevent races between:
+    /// * The background thread completes
+    /// * A did change notification coming in
+    /// * storing this struct on `Session`
+    ///
+    /// The revision helps us detect that a did change notification
+    /// happened in the meantime, so that we can reschedule the
+    /// workspace diagnostic request immediately.
+    pub(crate) revision: u64,
+}
+
+impl SuspendedWorkspaceDiagnosticRequest {
+    fn resume_if_revision_changed(self, current_revision: u64, client: &Client) -> Option<Self> {
+        if self.revision == current_revision {
+            return Some(self);
+        }
+
+        tracing::debug!("Resuming workspace diagnostics request after revision bump");
+        client.queue_action(Action::RetryRequest(lsp_server::Request {
+            id: self.id,
+            method: WorkspaceDiagnosticRequest::METHOD.to_string(),
+            params: self.params,
+        }));
+
+        None
+    }
+}
